library bucketeer;

import 'dart:io';

import 'package:flutter/services.dart';

import 'result.dart';

/// Bucketeer Flutter SDK
class Bucketeer {
  const Bucketeer._();

  static late final Bucketeer instance = Bucketeer._();

  static const _emptyResponse = '';

  static const MethodChannel _channel =
      MethodChannel('jp.bucketeer.plugin/flutter');

  Future<Result<void>> initialize({
    required String apiKey,
    required String endpoint,
    required String featureTag,
    bool debugging = false,
    int logSendingIntervalMillis = 60000,
    int logSendingMaxBatchQueueCount = 50,
    int pollingEvaluationIntervalMillis = 600000,
  }) async {
    return resultGuard(
      'initialize',
      await _channel.invokeMethod('initialize', {
        'apiKey': apiKey,
        'endpoint': endpoint,
        'featureTag': featureTag,
        'debugging': debugging,
        'logSendingIntervalMillis': logSendingIntervalMillis,
        'logSendingMaxBatchQueueCount': logSendingMaxBatchQueueCount,
        'pollingEvaluationIntervalMillis': pollingEvaluationIntervalMillis,
      }),
    );
  }

  Future<Result<void>> setUser(String userId,
      {Map<String, String> userMap = const {}}) async {
    return resultGuard(
      'setUser',
      await _channel.invokeMethod('setUser', {
        'userId': userId,
        ...userMap,
      }),
    );
  }

  Future<Result<String>> getStringVariation(String featureId,
      {String defaultValue = ''}) async {
    return resultGuard(
      'getStringVariation',
      await _channel.invokeMethod('getStringVariation', {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<int>> getIntVariation(String featureId,
      {int defaultValue = 0}) async {
    return resultGuard<int>(
      'getIntVariation',
      await _channel.invokeMethod('getIntVariation', {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<double>> getDoubleVariation(String featureId,
      {double defaultValue = 0.0}) async {
    return resultGuard<double>(
      'getDoubleVariation',
      await _channel.invokeMethod('getDoubleVariation', {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<bool>> getBoolVariation(String featureId,
      {bool defaultValue = false}) async {
    return resultGuard<bool>(
        'getBoolVariation',
        await _channel.invokeMethod('getBoolVariation', {
          'featureId': featureId,
          'defaultValue': defaultValue,
        }));
  }

  Future<Result<void>> track(String goalId, {double value = 0.0}) async {
    return resultGuard(
      'track',
      await _channel.invokeMethod('track', {
        'goalId': goalId,
        'value': value,
      }),
    );
  }

  Future<Result<void>> start() async {
    // Only run on Android
    if (Platform.isIOS) return Future.value(Result.success());
    return resultGuard(
      'start',
      await _channel.invokeMethod('start'),
    );
  }

  Future<Result<void>> stop() async {
    // Only run on Android
    if (Platform.isIOS) return Future.value(Result.success());
    return resultGuard(
      'stop',
      await _channel.invokeMethod('stop'),
    );
  }

  Result<T> resultGuard<T>(String method, dynamic result) {
    if (result['status']) {
      return Result<T>.success(data: result['response'] ?? _emptyResponse);
    } else {
      return Result.failure(result['errorMessage']);
    }
  }
}
